﻿#Requires -RunAsAdministrator
#Requires -Version 7


# Copyright (c) 2022 Vossloh-Schwabe Deutschland GmbH
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal 
# in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
# the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE 
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
# OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

$langw_whl = 'install_files/vs_lan_gateway_server-1.13-py3-none-any.whl'
$sslpsk2_whl = 'install_files/sslpsk2-1.0.3-cp39-cp39-win_amd64.whl'
$ErrorActionPreference = 'Stop'
$ProgressPreference = 'SilentlyContinue'
$oldpropre = $Global:ProgressPreference
$Global:ProgressPreference = 'SilentlyContinue'

Function Test-CommandExists {
    Param ($command)
    $oldPreference = $ErrorActionPreference
    $ErrorActionPreference = "stop"
    try { if (Get-Command $command) { RETURN $true } }
    Catch { RETURN $false }
    Finally { $ErrorActionPreference = $oldPreference }
} #end function test-CommandExists

$old_location = Get-Location
Set-Location $PSScriptRoot

if ($args.Count -gt 0 -and $args[0] -eq 'uninstall') {
    # Uninstall software
    do {
        Write-Host -ForegroundColor Yellow "Do you want to uninstall the server and all dependencies? [y/n] " -NoNewline
        $answer1 = Read-Host
    } until ($answer1.ToLower()[0] -in ('y', 'n'))
    if ($answer1.ToLower()[0] -eq 'n') {
        return
    }
	if (-not (Test-CommandExists nssm)) {
		Write-Host -ForegroundColor Red "Uninstallation is not possible as it is not proper installed yet! "
		return
	}

    # Remove all data
    Stop-Service -Name LanGatewayMariaDB -Force -ErrorAction SilentlyContinue | Out-Null
    nssm stop LanGatewayGrafana
    nssm remove LanGatewayGrafana confirm
    nssm stop LanGatewayService
    nssm remove LanGatewayService confirm
    Remove-Service LanGatewayMariaDB -ErrorAction SilentlyContinue | Out-Null
    if (Test-Path (Join-Path $Env:ALLUSERSPROFILE .vs_lan_gateway_server)) {
        Get-ChildItem (Join-Path $Env:ALLUSERSPROFILE .vs_lan_gateway_server) -Recurse | Remove-Item
    }
    if (Test-Path (Join-Path $Env:ALLUSERSPROFILE scoop)) {
        Get-ChildItem (Join-Path $Env:ALLUSERSPROFILE scoop) -Recurse -Force | ForEach-Object { $_.Attributes -= "ReadOnly"; $_.Attributes -= "Hidden"; $_.Attributes -= "System" }
        Get-ChildItem (Join-Path $Env:ALLUSERSPROFILE scoop) | Remove-Item -Recurse -Force
    }
    if (Test-Path (Join-Path $env:USERPROFILE scoop))
    {
        Get-ChildItem (Join-Path $Env:USERPROFILE scoop) -Recurse -Force | ForEach-Object { $_.Attributes -= "ReadOnly"; $_.Attributes -= "Hidden"; $_.Attributes -= "System" }
        Get-ChildItem (Join-Path $Env:USERPROFILE scoop) | Remove-Item -Recurse -Force
    }
	Remove-NetFirewallRule -DisplayName "LAN Gateway TCP" | Out-Null
	Remove-NetFirewallRule -DisplayName "LAN Gateway UDP" | Out-Null
	Remove-NetFirewallRule -DisplayName "Grafana" | Out-Null	
    Write-Host "Uninstallation successful."
    return
}

# Check whether we are on a workstation, server or domain controller
switch ((Get-CimInstance -ClassName Win32_OperatingSystem).ProductType) {
    1 {
        do {
            Write-Host -ForegroundColor Yellow "WARNING: Installing the server on a non-server operating system is NOT recommended. Are you sure you want to continue? [y/n] " -NoNewline
            $answer1 = Read-Host
        } until ($answer1.ToLower()[0] -in ('y', 'n'))
        if ($answer1.ToLower()[0] -eq 'n') {
            return
        }
    }
    2 {
        Write-Host -ForegroundColor Red "ALERT: This software should NOT be installed on a domain controller!"
        return
    }
    Default {}
}



$json = @{}
$dbms = $null
$config_file = Join-Path $Env:ALLUSERSPROFILE .vs_lan_gateway_server config.json

Write-Host "Database Setup"

while ($dbms -notin ("influxdb", "mariadb", "i", "m")) {
    $dbms = Read-Host -Prompt "Enter the database system you want to use ((i)nfluxdb/(m)ariadb)"
}

if ($dbms -eq "i") {
    $dbms = "influxdb"
}
elseif ($dbms -eq "m") {
    $dbms = "mariadb"
}


if ($dbms -eq "mariadb") {
    $mariadb_dbname = Read-Host -Prompt "Enter the database name"
    do {
        $mariadb_username = Read-Host -Prompt "Enter the database username (root must not be used)"
    } while ($mariadb_username -eq "root")
    $mariadb_password = Read-Host -Prompt "Enter the database password" -MaskInput
}


if (Test-Path $config_file) {
    do {
        $config_overwrite_answer = Read-Host -Prompt "There is already a server configuration. Do you want to overwrite it with the MariaDB settings? (You can also configure it later on the server configuration page) (y/n)"
    } until ($config_overwrite_answer -in ("y", "n"))

    if ($config_overwrite_answer -eq "y") {
        $json.mariadb = @{
            enabled  = $true;
            username = $mariadb_username;
            dbname   = $mariadb_dbname;
            password = $mariadb_password
        }
    }
}
else {
    $config_overwrite_answer = "y"
    $json.mariadb = @{
        enabled  = $true;
        username = $mariadb_username;
        dbname   = $mariadb_dbname;
        password = $mariadb_password
    }
    New-Item (Join-Path $Env:ALLUSERSPROFILE .vs_lan_gateway_server) -ItemType Directory -Force | Out-Null
	New-Item (Join-Path $Env:ALLUSERSPROFILE .vs_lan_gateway_server dashboards) -ItemType Directory -Force | Out-Null
}
Write-Host "Database Setup done"

Write-Host "Installing dependencies for Windows..."
if (-not (Test-CommandExists scoop)) {
    Invoke-WebRequest -useb get.scoop.sh -OutFile (Join-Path $env:TEMP scoop.ps1) | Out-Null # install scoop
    . (Join-Path $env:TEMP scoop.ps1) -RunAsAdmin
}

Write-Host "Install and update packages"
scoop install sudo git --global
# Workaround Scoop bug
scoop bucket rm main
scoop bucket add main
# end Workaround
scoop update
scoop bucket add versions
scoop install python39 --global
scoop bucket add extras
scoop install $dbms grafana nssm --global
pip install PyMySQL
#pip install sslpsk2==1.0.1
pip install $sslpsk2_whl
pip install $langw_whl
pip install $dbms
Remove-Item -Path (Join-Path $Env:ALLUSERSPROFILE scoop apps mariadb current data *) -Filter * -Recurse -Force
if ($null -eq (get-service -Name LanGatewayMariaDB -ErrorAction SilentlyContinue)) {
    mysql_install_db --service=LanGatewayMariaDB --password=mariadb
}
if ($null -eq (get-service -Name LanGatewayService -ErrorAction SilentlyContinue)) {
    nssm install LanGatewayService "`"$(Join-Path $Env:ALLUSERSPROFILE scoop apps python39 current Scripts vs_lan_gateway_server.exe)`""
    nssm set LanGatewayService AppParameters "--daemon"
    nssm start LanGatewayService
}
Start-Service LanGatewayMariaDB
if ($null -eq (get-service -Name LanGatewayGrafana -ErrorAction SilentlyContinue)) {
    nssm install LanGatewayGrafana "`"$(Join-Path $Env:ALLUSERSPROFILE scoop shims grafana-server.exe)`""
    nssm set LanGatewayGrafana AppDirectory "`"$(Join-Path $Env:ALLUSERSPROFILE scoop apps grafana current bin)`""
	$cmd = Join-Path $Env:ALLUSERSPROFILE scoop apps grafana current bin grafana-cli.exe
	& $cmd plugins install ryantxu-ajax-panel | Out-Null 
    nssm start LanGatewayGrafana
}

if ($dbms -eq "mariadb") {
    $my_ini_path = Join-Path $Env:ALLUSERSPROFILE scoop apps mariadb current data my.ini
    if (Test-Path $my_ini_path) {
        Copy-Item $my_ini_path ($my_ini_path + ".000001") -Force
    }
    else {
        New-Item (Join-Path $Env:ALLUSERSPROFILE scoop apps mariadb current data) -ItemType Directory | Out-Null
    }
    Write-Output (@"
[mysql]
user=root
password=mariadb
"@) | Out-File -Append $my_ini_path -Force
    (Get-Content .\install_files\init.sql).Replace("<database>", "$mariadb_dbname").Replace("<username>", "$mariadb_username").Replace("<password>", "$mariadb_password") | mysql
    if (Test-Path ($my_ini_path + ".000001")) {
        Move-Item ($my_ini_path + ".000001") $my_ini_path -Force
    }
}

Clear-Host

if ($config_overwrite_answer -eq "y") {
    Write-Host 'Please enter credentials for the web configuration'

    $web_username = Read-Host -Prompt "Username"
    $web_password = Read-Host -Prompt "Password" -MaskInput
    $web_password = (python3 -c "from werkzeug.security import generate_password_hash; print(generate_password_hash(\`"$web_password\`"))")
    $json.web = @{
        username = $web_username;
        password = $web_password
    }
    ConvertTo-Json $json | Out-File $config_file
}
New-NetFirewallRule -DisplayName "LAN Gateway TCP" -Enabled True -Direction Inbound -LocalPort 31460, 31461 -Action Allow -Protocol TCP -Profile Any | Out-Null
New-NetFirewallRule -DisplayName "LAN Gateway UDP" -Enabled True -Direction Inbound -LocalPort 31461 -Action Allow -Protocol UDP -Profile Any | Out-Null
New-NetFirewallRule -DisplayName "Grafana" -Enabled True -Direction Inbound -LocalPort 3000 -Action Allow -Protocol TCP -Profile Any | Out-Null

New-Item (Join-Path $Env:ALLUSERSPROFILE scoop persist grafana conf provisioning datasources) -ItemType Directory -Force | Out-Null
New-Item (Join-Path $Env:ALLUSERSPROFILE scoop persist grafana conf provisioning dashboards) -ItemType Directory -Force | Out-Null

if ($dbms -eq "mariadb") {
    (Get-Content .\install_files\mariadb.yaml).Replace("<database>", "$mariadb_dbname").Replace("<username>", "$mariadb_username").Replace("<password>", "$mariadb_password") | Set-Content (Join-Path $Env:ALLUSERSPROFILE scoop persist grafana conf provisioning datasources mariadb.yaml)
}
else {
    Copy-Item .\install_files\influxdb.yaml (Join-Path $Env:ALLUSERSPROFILE scoop persist grafana conf provisioning datasources influxdb.yaml)
}

Copy-Item .\install_files\dashboard_win.yaml (Join-Path $Env:ALLUSERSPROFILE scoop persist grafana conf provisioning dashboards dashboard.yaml)

nssm restart LanGatewayGrafana

$Global:ProgressPreference = $oldpropre

Write-Host ''
Write-Host ''
Write-Host '################################################################################################'
Write-Host '# Setup successful.                                                                            #'
Write-Host '# The configuration website is reachable at following addresses:                               #'
Get-NetIPAddress | Where-Object { -not $_.IPAddress.Contains('%') } | ForEach-Object { if ($_.AddressFamily -eq 'IPv6') { Write-Host ('  http://[' + $_.IPAddress + ']:31460/') } else { Write-Host ('  http://' + $_.IPAddress + ':31460/') } }
Write-Host '# Configuration website after starting: http://<ip of device>:31460/                           #'
Write-Host '# Some configurations on the configuration page are needed                                     #'
Write-Host '# before the server can work completely.                                                       #'
Write-Host '################################################################################################'
Set-Location $old_location